==> itkspec-1.2.1-py3-none-any.whl: itkspec/__main__.py <==
import os
import itkdb
import pandas as pd
from fastapi import FastAPI, HTTPException
from pydantic import BaseModel, Field
from typing import Optional, List, Dict, Any
from ReadSpec import getSpec, getSpecList
import json

app = FastAPI()

# Model for the input of parameters
class SpecDataInput(BaseModel):
    accessCode1: str = Field(default=None,examples=["my_access_code1"])
    accessCode2: str = Field(default=None,examples=["my_access_code2"])
    project: str = Field(default=None,examples=["P"])
    componentType: str = Field(default=None,examples=["PCB"])
    testType: str = Field(default=None,examples=["METROLOGY"])
    stage: str = Field(default=None,examples=["PCB_RECEPTION"])
    parameter: Optional[str] = Field(default=None,examples=["BOW1"])

# Model for the output of parameters
class SpecDataOutput(BaseModel):
    project: Optional[str] = None
    componentType: Optional[str] = None
    testType: Optional[str] = None
    stage: Optional[str] = None
    parameter: List[str] = None
    specList: Dict[str, Dict[str, Any]] = None
    

#################
### FUNCTIONS ###
#################

def help():
    print("Welcome to itk-spec. Here's a list of usable functions with their corresponding input data types: retrieve_spec(dictionary, string), retrieve_spec_list(dictionary, string), help_api(), and help().")
    print("You can choose between manually inputting the data or reading it from a JSON file. To read from a JSON file, call the function with the parameter path='path_to_your_json'.")
    print("Example of manual data input: {'accessCode1':'your_access_code1', 'accessCode2':'your_access_code2', 'project':'P', 'componentType':'PCB', 'testType':'METROLOGY', 'stage':'PCB_RECEPTION', 'parameter':'BOW1'}")

### Input function receives a dictionary and turns it into SpecDataInput object
def input(kwargs=None):
    if kwargs is None:
        print("No input data provided. please call this function with a dictionary of parameters. Example:")
        print("{'accessCode1':'your_access_code1', 'accessCode2':'your_access_code2', 'project':'P', 'componentType':'PCB', 'testType':'METROLOGY', 'stage':'PCB_RECEPTION', 'parameter':'BOW1'}")
    else:
        input_data = SpecDataInput(
        accessCode1=kwargs["accessCode1"],
        accessCode2=kwargs["accessCode2"],
        project=kwargs["project"],
        componentType=kwargs["componentType"],
        testType=kwargs["testType"],
        stage=kwargs["stage"],
        parameter=kwargs["parameter"]
        )
        return input_data

### Read JSON function reads a JSON file and returns its content
def read_json(path):
    print("searching json file...")
    with open(path, 'r') as f:
        data = json.load(f)
    print(data)
    return data
### Authenticate user function receives two access codes and returns the user variable
def authenticate_user(accessCode1: str, accessCode2: str):
    user = itkdb.core.User(access_code1=accessCode1, access_code2=accessCode2)
    try:
        user.authenticate()
    except Exception as e:
        print(e)
        raise HTTPException(status_code=401, detail="Authentication failed")
    return user

### Single specification retrieval
def retrieve_spec(kwargs=None, path=None):
    if type(kwargs) is Dict:
        kwargs=input(kwargs)
        kwargs_dict = kwargs.model_dump()
        authenticate_user(kwargs_dict["accessCode1"], kwargs_dict["accessCode2"])
        query = getSpec(**kwargs_dict)
        output = SpecDataOutput(
            project=kwargs_dict["project"],
            componentType=kwargs_dict["componentType"],
            testType=kwargs_dict["testType"],
            stage=kwargs_dict["stage"],
            parameter=[query["parameter"]],
            specList={query["parameter"]: query["spec"]}
        )
        return output
    elif type(path) is str:
        data = read_json(path)
        authenticate_user(data["accessCode1"], data["accessCode2"])
        query = getSpec(**data)
        output = SpecDataOutput(
            project=data["project"],
            componentType=data["componentType"],
            testType=data["testType"],
            stage=data["stage"],
            parameter=[query["parameter"]],
            specList={query["parameter"]: query["spec"]}
        )
        return output

### Multiple specifications retrieval
def retrieve_spec_list(kwargs=None, path=None):
    if type(kwargs) is Dict:
        kwargs=input(kwargs)
        kwargs_dict = kwargs.model_dump()
        authenticate_user(kwargs_dict["accessCode1"], kwargs_dict["accessCode2"])
        query = getSpecList(**kwargs_dict)
        output = SpecDataOutput(
            project=kwargs_dict["project"],
            componentType=kwargs_dict["componentType"],
            testType=kwargs_dict["testType"],
            stage=kwargs_dict["stage"],
            parameter=query["parameter"],
            specList={x: y | {"associatedParam": []} for x, y in zip(query["parameter"], query["spec"])}
        )
        return output
    elif type(path) is str:
        data = read_json(path)
        authenticate_user(data["accessCode1"], data["accessCode2"])
        query = getSpec(**data)
        data=kwargs_dict
        output = SpecDataOutput(
            project=kwargs_dict["project"],
            componentType=kwargs_dict["componentType"],
            testType=kwargs_dict["testType"],
            stage=kwargs_dict["stage"],
            parameter=query["parameter"],
            specList={x: y | {"associatedParam": []} for x, y in zip(query["parameter"], query["spec"])}
        )
        return output
def help_api():
    return {
        "message": "Welcome to itk-spec",
        "endpoints": {
            "/spec": {
                "method": "GET",
                "description": "Get specification for a single parameter",
                "parameters": {
                    "project": "Project name",
                    "componentType": "Component type",
                    "testType": "Test type",
                    "stage": "Stage",
                    "parameter": "Parameter"
                }
            },
            "/speclist": {
                "method": "GET",
                "description": "Get specifications for multiple parameters",
                "parameters": {
                    "project": "Project name",
                    "componentType": "Component type",
                    "testType": "Test type",
                    "stage": "Stage",
                }
            },
            "/modules": {
                "method": "GET",
                "description": "Get all modules"
            },
            "/modules/{module_id}": {
                "method": "GET",
                "description": "Get a specific module by ID"
            }
        }
    }

#################
### ENDPOINTS ###
#################


### Single specification retrieval
@app.post("/spec")
async def specRetrieval(kwargs: SpecDataInput):
    kwargs_dict=kwargs.model_dump()

    # auth
    user = itkdb.core.User(access_code1=kwargs_dict["accessCode1"], access_code2=kwargs_dict["accessCode2"])
    try:
        user.authenticate()
    except Exception as e:
        print(e)
        raise HTTPException(status_code=401, detail="Authentication failed")
    
    # function
    query = getSpec(**kwargs_dict)

    # output structure
    output = SpecDataOutput(
        project=kwargs_dict["project"],
        componentType=kwargs_dict["componentType"],
        testType=kwargs_dict["testType"],
        stage=kwargs_dict["stage"],
        parameter=[query["parameter"]],
        specList={query["parameter"]:query["spec"]}
    )

    return output


### Multiple specifications retrieval
@app.post("/speclist")
async def specListRetrieval(kwargs: SpecDataInput):
    kwargs_dict=kwargs.model_dump()

    # auth
    user = itkdb.core.User(access_code1=kwargs_dict["accessCode1"], access_code2=kwargs_dict["accessCode2"])
    try:
        user.authenticate()
    except Exception as e:
        print(e)
        raise HTTPException(status_code=401, detail="Authentication failed")
    
    # function
    query = getSpecList(**kwargs_dict)

    # output structure
    output = SpecDataOutput(
        project=kwargs_dict["project"],
        componentType=kwargs_dict["componentType"],
        testType=kwargs_dict["testType"],
        stage=kwargs_dict["stage"],
        parameter=query["parameter"],
        specList={x : y | {"associatedParam": []} for x, y in zip(query["parameter"], query["spec"])}
    )
    return output


# Example endpoint to read modules
@app.get("/modules")
def read_modules():
    csv_folder_path = "/spec_files"

    if not os.path.exists(csv_folder_path):
        raise HTTPException(status_code=404, detail="CSV files not found")
    try:
        df = pd.read_csv(csv_folder_path)
        return df.to_dict(orient="records")
    except Exception as e:
        raise HTTPException(status_code=500, detail=str(e))


# Example endpoint to read a specific module by ID
@app.get("/modules/{module_id}")
def read_module(module_id: int):
    # treating the module id input
    if module_id[-4:] != ".csv":
        module_id = module_id + ".csv"

    # csv file path if the working directory is the root of the project
    csv_file_path = f"/spec_files/{module_id}"

    if not os.path.exists(csv_file_path):
        raise HTTPException(status_code=404, detail="CSV file not found")
    try:
        df = pd.read_csv(csv_file_path)
        if module_id >= len(df) or module_id < 0:
            raise HTTPException(status_code=404, detail="Module ID not found")
        return df.iloc[module_id].to_dict()
    except Exception as e:
        raise HTTPException(status_code=500, detail=str(e))

@app.get("/help")
def api_help():
    return help_api()

if __name__ == "__main__":
    #retrieve_spec({'accessCode1':'flaflu2002!', 'accessCode2':'Myestu0511!', 'project':'P', 'componentType':'PCB', 'testType':'METROLOGY', 'stage':'PCB_RECEPTION', 'parameter':'BOW1'})
    retrieve_spec(path='C:/itk/itk-pdb-specifica/test.json')
    import uvicorn
    uvicorn.run(app, host="0.0.0.0", port=8000, log_level="info")

==> itkspec-1.2.5-py3-none-any.whl: itkspec/__main__.py <==
import os
import itkdb
import pandas as pd
from fastapi import FastAPI, HTTPException
from pydantic import BaseModel, Field
from typing import Optional, List, Dict, Any
from .ReadSpec import getSpec, getSpecList
import json

app = FastAPI()

# Model for the input of parameters
class SpecDataInput(BaseModel):
    accessCode1: str = Field(default=None,examples=["my_access_code1"])
    accessCode2: str = Field(default=None,examples=["my_access_code2"])
    project: str = Field(default=None,examples=["P"])
    componentType: str = Field(default=None,examples=["PCB"])
    testType: str = Field(default=None,examples=["METROLOGY"])
    stage: str = Field(default=None,examples=["PCB_RECEPTION"])
    parameter: Optional[str] = Field(default=None,examples=["BOW1"])

# Model for the output of parameters
class SpecDataOutput(BaseModel):
    project: Optional[str] = None
    componentType: Optional[str] = None
    testType: Optional[str] = None
    stage: Optional[str] = None
    parameter: List[str] = None
    specList: Dict[str, Dict[str, Any]] = None
    

#################
### FUNCTIONS ###
#################

def help():
    print("Welcome to itk-spec. Here's a list of usable functions with their corresponding input data types: retrieve_spec(dictionary, string), retrieve_spec_list(dictionary, string), help_api(), and help().")
    print("You can choose between manually inputting the data or reading it from a JSON file. To read from a JSON file, call the function with the parameter path='path_to_your_json'.")
    print("Example of manual data input: {'accessCode1':'your_access_code1', 'accessCode2':'your_access_code2', 'project':'P', 'componentType':'PCB', 'testType':'METROLOGY', 'stage':'PCB_RECEPTION', 'parameter':'BOW1'}")

### Input function receives a dictionary and turns it into SpecDataInput object
def input(kwargs=None):
    if kwargs is None:
        print("No input data provided. please call this function with a dictionary of parameters. Example:")
        print("{'accessCode1':'your_access_code1', 'accessCode2':'your_access_code2', 'project':'P', 'componentType':'PCB', 'testType':'METROLOGY', 'stage':'PCB_RECEPTION', 'parameter':'BOW1'}")
    else:
        input_data = SpecDataInput(
        accessCode1=kwargs["accessCode1"],
        accessCode2=kwargs["accessCode2"],
        project=kwargs["project"],
        componentType=kwargs["componentType"],
        testType=kwargs["testType"],
        stage=kwargs["stage"],
        parameter=kwargs["parameter"]
        )
        return input_data

### Read JSON function reads a JSON file and returns its content
def read_json(path):
    print("searching json file...")
    with open(path, 'r') as f:
        data = json.load(f)
    print(data)
    return data
### Authenticate user function receives two access codes and returns the user variable
def authenticate_user(accessCode1: str, accessCode2: str):
    user = itkdb.core.User(access_code1=accessCode1, access_code2=accessCode2)
    try:
        user.authenticate()
    except Exception as e:
        print(e)
        raise HTTPException(status_code=401, detail="Authentication failed")
    return user

### Single specification retrieval
def retrieve_spec(kwargs=None, path=None):
    if type(kwargs) is Dict:
        kwargs=input(kwargs)
        kwargs_dict = kwargs.model_dump()
        authenticate_user(kwargs_dict["accessCode1"], kwargs_dict["accessCode2"])
        query = getSpec(**kwargs_dict)
        output = SpecDataOutput(
            project=kwargs_dict["project"],
            componentType=kwargs_dict["componentType"],
            testType=kwargs_dict["testType"],
            stage=kwargs_dict["stage"],
            parameter=[query["parameter"]],
            specList={query["parameter"]: query["spec"]}
        )
        return output
    elif type(path) is str:
        data = read_json(path)
        authenticate_user(data["accessCode1"], data["accessCode2"])
        query = getSpec(**data)
        output = SpecDataOutput(
            project=data["project"],
            componentType=data["componentType"],
            testType=data["testType"],
            stage=data["stage"],
            parameter=[query["parameter"]],
            specList={query["parameter"]: query["spec"]}
        )
        return output

### Multiple specifications retrieval
def retrieve_spec_list(kwargs=None, path=None):
    if type(kwargs) is Dict:
        kwargs=input(kwargs)
        kwargs_dict = kwargs.model_dump()
        authenticate_user(kwargs_dict["accessCode1"], kwargs_dict["accessCode2"])
        query = getSpecList(**kwargs_dict)
        output = SpecDataOutput(
            project=kwargs_dict["project"],
            componentType=kwargs_dict["componentType"],
            testType=kwargs_dict["testType"],
            stage=kwargs_dict["stage"],
            parameter=query["parameter"],
            specList={x: y | {"associatedParam": []} for x, y in zip(query["parameter"], query["spec"])}
        )
        return output
    elif type(path) is str:
        data = read_json(path)
        authenticate_user(data["accessCode1"], data["accessCode2"])
        query = getSpec(**data)
        data=kwargs_dict
        output = SpecDataOutput(
            project=kwargs_dict["project"],
            componentType=kwargs_dict["componentType"],
            testType=kwargs_dict["testType"],
            stage=kwargs_dict["stage"],
            parameter=query["parameter"],
            specList={x: y | {"associatedParam": []} for x, y in zip(query["parameter"], query["spec"])}
        )
        return output
def help_api():
    return {
        "message": "Welcome to itk-spec",
        "endpoints": {
            "/spec": {
                "method": "GET",
                "description": "Get specification for a single parameter",
                "parameters": {
                    "project": "Project name",
                    "componentType": "Component type",
                    "testType": "Test type",
                    "stage": "Stage",
                    "parameter": "Parameter"
                }
            },
            "/speclist": {
                "method": "GET",
                "description": "Get specifications for multiple parameters",
                "parameters": {
                    "project": "Project name",
                    "componentType": "Component type",
                    "testType": "Test type",
                    "stage": "Stage",
                }
            },
            "/modules": {
                "method": "GET",
                "description": "Get all modules"
            },
            "/modules/{module_id}": {
                "method": "GET",
                "description": "Get a specific module by ID"
            }
        }
    }

#################
### ENDPOINTS ###
#################


### Single specification retrieval
@app.post("/spec")
async def specRetrieval(kwargs: SpecDataInput):
    kwargs_dict=kwargs.model_dump()

    # auth
    user = itkdb.core.User(access_code1=kwargs_dict["accessCode1"], access_code2=kwargs_dict["accessCode2"])
    try:
        user.authenticate()
    except Exception as e:
        print(e)
        raise HTTPException(status_code=401, detail="Authentication failed")
    
    # function
    query = getSpec(**kwargs_dict)

    # output structure
    output = SpecDataOutput(
        project=kwargs_dict["project"],
        componentType=kwargs_dict["componentType"],
        testType=kwargs_dict["testType"],
        stage=kwargs_dict["stage"],
        parameter=[query["parameter"]],
        specList={query["parameter"]:query["spec"]}
    )

    return output


### Multiple specifications retrieval
@app.post("/speclist")
async def specListRetrieval(kwargs: SpecDataInput):
    kwargs_dict=kwargs.model_dump()

    # auth
    user = itkdb.core.User(access_code1=kwargs_dict["accessCode1"], access_code2=kwargs_dict["accessCode2"])
    try:
        user.authenticate()
    except Exception as e:
        print(e)
        raise HTTPException(status_code=401, detail="Authentication failed")
    
    # function
    query = getSpecList(**kwargs_dict)

    # output structure
    output = SpecDataOutput(
        project=kwargs_dict["project"],
        componentType=kwargs_dict["componentType"],
        testType=kwargs_dict["testType"],
        stage=kwargs_dict["stage"],
        parameter=query["parameter"],
        specList={x : y | {"associatedParam": []} for x, y in zip(query["parameter"], query["spec"])}
    )
    return output


# Example endpoint to read modules
@app.get("/modules")
def read_modules():
    csv_folder_path = "/spec_files"

    if not os.path.exists(csv_folder_path):
        raise HTTPException(status_code=404, detail="CSV files not found")
    try:
        df = pd.read_csv(csv_folder_path)
        return df.to_dict(orient="records")
    except Exception as e:
        raise HTTPException(status_code=500, detail=str(e))


# Example endpoint to read a specific module by ID
@app.get("/modules/{module_id}")
def read_module(module_id: int):
    # treating the module id input
    if module_id[-4:] != ".csv":
        module_id = module_id + ".csv"

    # csv file path if the working directory is the root of the project
    csv_file_path = f"/spec_files/{module_id}"

    if not os.path.exists(csv_file_path):
        raise HTTPException(status_code=404, detail="CSV file not found")
    try:
        df = pd.read_csv(csv_file_path)
        if module_id >= len(df) or module_id < 0:
            raise HTTPException(status_code=404, detail="Module ID not found")
        return df.iloc[module_id].to_dict()
    except Exception as e:
        raise HTTPException(status_code=500, detail=str(e))

@app.get("/help")
def api_help():
    return help_api()

if __name__ == "__main__":
    import uvicorn
    uvicorn.run(app, host="0.0.0.0", port=8000, log_level="info")
